@@ -1,6 +1,7 @@
 #pragma once
 
 #include "../base/string_view.h"
+#include "types.h"
 
 #include <list>
 #include <stack>
@@ -16,15 +17,20 @@ struct TypeAst {
         Number,
         Terminal,
         Tuple,
+        Enum
     };
 
     /// Type's category.
     Meta meta;
+    Type::Code code;
     /// Type's name.
-    StringView name;
-    /// Size of type's instance.  For fixed-width types only.
-    size_t size = 0;
+    /// Need to cache TypeAst, so can't use StringView for name.
+    std::string name;
+    /// Value associated with the node,
+    /// used for fixed-width types and enum values.
+    int64_t value = 0;
     /// Subelements of the type.
+    /// Used to store enum's names and values as well.
     std::list<TypeAst> elements;
 };
 
@@ -63,4 +69,7 @@ private:
     std::stack<TypeAst*> open_elements_;
 };
 
+
+const TypeAst* ParseTypeName(const std::string& type_name);
+
 }
