@@ -7,6 +7,10 @@
 #include "SpookyV2.h"
 #include <cxxabi.h>
 
+#include "MySQL_LDAP_Authentication.hpp"
+
+extern MySQL_LDAP_Authentication* GloMyLdapAuth;
+
 static void term_handler(int sig) {
   proxy_warning("Received TERM signal: shutdown in progress...\n");
 #ifdef DEBUG
@@ -367,6 +371,13 @@ uint64_t ProxySQL_GlobalVariables::generate_global_checksum() {
 		myhash.Update(v->checksum,strlen(v->checksum));
 		myhash.Update(&v->version,sizeof(v->version));
 	}
+	if (GloMyLdapAuth) {
+		v = &checksums_values.ldap_variables;
+		if (v->version) {
+			myhash.Update(v->checksum,strlen(v->checksum));
+			myhash.Update(&v->version,sizeof(v->version));
+		}
+	}
 	uint64_t h1, h2;
 	myhash.Final(&h1, &h2);
 	h1 = h1/2; // ugly way to make it signed within LLONG_MAX
