@@ -126,8 +126,10 @@ void Variable::fill_server_internal_session(json &j, int conn_num, int idx) {
 					j["backends"][conn_num]["conn"][mysql_tracked_variables[idx].internal_variable_name] = value;
 				}
 			} else {
+				// LCOV_EXCL_START
 				proxy_error("Cannot find charset [%s] for variables %d\n", value, idx);
 				assert(0);
+				// LCOV_EXCL_STOP
 			}
 		} else {
 			j["backends"][conn_num]["conn"][mysql_tracked_variables[idx].internal_variable_name] = std::string((ci && ci->csname)?ci->csname:"");
@@ -174,8 +176,10 @@ void Variable::fill_client_internal_session(json &j, int idx) {
 					j["conn"][mysql_tracked_variables[idx].internal_variable_name] = value;
 				}
 			} else {
+				// LCOV_EXCL_START
 				proxy_error("Cannot find charset [%s] for variables %d\n", value, idx);
 				assert(0);
+				// LCOV_EXCL_STOP
 			}
 		} else {
 			j["conn"][mysql_tracked_variables[idx].internal_variable_name] = (ci && ci->csname)?ci->csname:"";
@@ -204,8 +208,6 @@ void Variable::fill_client_internal_session(json &j, int idx) {
 	}
 }
 
-#define PROXYSQL_USE_RESULT
-
 static int
 mysql_status(short event, short cont) {
 	int status= 0;
@@ -654,10 +656,17 @@ void MySQL_Connection::connect_start() {
 		} else {
 			mysql_options4(mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "proxysql_sha1", "unknown");
 		}
-		mysql_options4(mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "mysql_bug_102266", "ProxySQL is sending a lot of data to MySQL server using CLIENT_CONNECT_ATTRS in order to not hit MySQL bug https://bugs.mysql.com/bug.php?id=102266 . See also https://github.com/sysown/proxysql/issues/3276");
+		mysql_options4(mysql, MYSQL_OPT_CONNECT_ATTR_ADD, "mysql_bug_102266", "Avoid MySQL bug https://bugs.mysql.com/bug.php?id=102266 , https://github.com/sysown/proxysql/issues/3276");
 	}
 	if (parent->use_ssl) {
-		mysql_ssl_set(mysql, mysql_thread___ssl_p2s_key, mysql_thread___ssl_p2s_cert, mysql_thread___ssl_p2s_ca, NULL, mysql_thread___ssl_p2s_cipher);
+		mysql_ssl_set(mysql,
+				mysql_thread___ssl_p2s_key,
+				mysql_thread___ssl_p2s_cert,
+				mysql_thread___ssl_p2s_ca,
+				mysql_thread___ssl_p2s_capath,
+				mysql_thread___ssl_p2s_cipher);
+		mysql_options(mysql, MYSQL_OPT_SSL_CRL, mysql_thread___ssl_p2s_crl);
+		mysql_options(mysql, MYSQL_OPT_SSL_CRLPATH, mysql_thread___ssl_p2s_crlpath);
 	}
 	unsigned int timeout= 1;
 	const char *csname = NULL;
@@ -674,8 +683,10 @@ void MySQL_Connection::connect_start() {
 		c = proxysql_find_charset_name(mysql_thread___default_variables[SQL_CHARACTER_SET]);
 
 	if (!c) {
+		// LCOV_EXCL_START
 		proxy_error("Not existing charset number %s\n", mysql_thread___default_variables[SQL_CHARACTER_SET]);
 		assert(0);
+		// LCOV_EXCL_STOP
 	}
 	{
 		/* We are connecting to backend setting charset in mysql_options.
@@ -730,6 +741,23 @@ void MySQL_Connection::connect_start() {
 		mysql->options.client_flag |= CLIENT_DEPRECATE_EOF;
 	}
 
+	if (myds != NULL) {
+		if (myds->sess != NULL) {
+			if (myds->sess->session_fast_forward == true) { // this is a fast_forward connection
+				assert(myds->sess->client_myds != NULL);
+				MySQL_Connection * c = myds->sess->client_myds->myconn;
+				assert(c != NULL);
+				mysql->options.client_flag &= ~(CLIENT_DEPRECATE_EOF); // we disable it by default
+				// if both client_flag and server_capabilities (used for client) , set CLIENT_DEPRECATE_EOF
+				if (c->options.client_flag & CLIENT_DEPRECATE_EOF) {
+					if (c->options.server_capabilities & CLIENT_DEPRECATE_EOF) {
+						mysql->options.client_flag |= CLIENT_DEPRECATE_EOF;
+					}
+				}
+			}
+		}
+	}
+
 	char *auth_password=NULL;
 	if (userinfo->password) {
 		if (userinfo->password[0]=='*') { // we don't have the real password, let's pass sha1
@@ -845,8 +873,10 @@ void MySQL_Connection::set_names_start() {
 	PROXY_TRACE();
 	const MARIADB_CHARSET_INFO * c = proxysql_find_charset_nr(atoi(mysql_variables.client_get_value(myds->sess, SQL_CHARACTER_SET)));
 	if (!c) {
+		// LCOV_EXCL_START
 		proxy_error("Not existing charset number %u\n", atoi(mysql_variables.client_get_value(myds->sess, SQL_CHARACTER_SET)));
 		assert(0);
+		// LCOV_EXCL_STOP
 	}
 	async_exit_status = mysql_set_character_set_start(&interr,mysql, NULL, atoi(mysql_variables.client_get_value(myds->sess, SQL_CHARACTER_SET)));
 }
@@ -899,17 +929,18 @@ void MySQL_Connection::stmt_execute_start() {
 	if (_rc) {
 		proxy_error("mysql_stmt_bind_param() failed: %s", mysql_stmt_error(query.stmt));
 	}
-	//proxy_info("Calling mysql_stmt_execute_start, current state: %d\n", query.stmt->state);
+	// if for whatever reason the previous execution failed, state is left to an inconsistent value
+	// see bug #3547
+	// here we force the state to be MYSQL_STMT_PREPARED
+	// it is a nasty hack because we shouldn't change states that should belong to the library
+	// I am not sure if this is a bug in the backend library or not
+	query.stmt->state= MYSQL_STMT_PREPARED;
 	async_exit_status = mysql_stmt_execute_start(&interr , query.stmt);
-	//fprintf(stderr,"Current state: %d\n", query.stmt->state);
 }
 
 void MySQL_Connection::stmt_execute_cont(short event) {
 	proxy_debug(PROXY_DEBUG_MYSQL_PROTOCOL, 6,"event=%d\n", event);
-	//proxy_info("Calling mysql_stmt_execute_cont, current state: %d\n", query.stmt->state);
 	async_exit_status = mysql_stmt_execute_cont(&interr , query.stmt , mysql_status(event, true));
-	//proxy_info("mysql_stmt_execute_cont , ret=%d\n", async_exit_status);
-	//fprintf(stderr,"Current state: %d\n", query.stmt->state);
 }
 
 void MySQL_Connection::stmt_execute_store_result_start() {
@@ -922,6 +953,7 @@ void MySQL_Connection::stmt_execute_store_result_cont(short event) {
 	async_exit_status = mysql_stmt_store_result_cont(&interr , query.stmt , mysql_status(event, true));
 }
 
+#ifndef PROXYSQL_USE_RESULT
 void MySQL_Connection::store_result_start() {
 	PROXY_TRACE();
 	async_exit_status = mysql_store_result_start(&mysql_result, mysql);
@@ -931,6 +963,7 @@ void MySQL_Connection::store_result_cont(short event) {
 	proxy_debug(PROXY_DEBUG_MYSQL_PROTOCOL, 6,"event=%d\n", event);
 	async_exit_status = mysql_store_result_cont(&mysql_result , mysql , mysql_status(event, true));
 }
+#endif // PROXYSQL_USE_RESULT
 
 void MySQL_Connection::set_is_client() {
 	local_stmts->set_is_client(myds->sess);
@@ -986,7 +1019,7 @@ handler_again:
 			}
 			if (!ret_mysql) {
 				// always increase the counter
-				proxy_error("Failed to mysql_real_connect() on %s:%d , FD (Conn:%d , MyDS:%d) , %d: %s.\n", parent->address, parent->port, mysql->net.fd , myds->fd, mysql_errno(mysql), mysql_error(mysql));
+				proxy_error("Failed to mysql_real_connect() on %u:%s:%d , FD (Conn:%d , MyDS:%d) , %d: %s.\n", parent->myhgc->hid, parent->address, parent->port, mysql->net.fd , myds->fd, mysql_errno(mysql), mysql_error(mysql));
     		NEXT_IMMEDIATE(ASYNC_CONNECT_FAILED);
 			} else {
     		NEXT_IMMEDIATE(ASYNC_CONNECT_SUCCESSFUL);
@@ -1168,6 +1201,7 @@ handler_again:
 			break;
 
 		case ASYNC_STMT_EXECUTE_START:
+			PROXY_TRACE2();
 			stmt_execute_start();
 			__sync_fetch_and_add(&parent->queries_sent,1);
 			__sync_fetch_and_add(&parent->bytes_sent,query.stmt_meta->size);
@@ -1181,6 +1215,7 @@ handler_again:
 			}
 			break;
 		case ASYNC_STMT_EXECUTE_CONT:
+			PROXY_TRACE2();
 			stmt_execute_cont(event);
 			if (async_exit_status) {
 				next_event(ASYNC_STMT_EXECUTE_CONT);
@@ -1190,6 +1225,7 @@ handler_again:
 			break;
 
 		case ASYNC_STMT_EXECUTE_STORE_RESULT_START:
+			PROXY_TRACE2();
 			if (mysql_stmt_errno(query.stmt)) {
 				NEXT_IMMEDIATE(ASYNC_STMT_EXECUTE_END);
 			}
@@ -1231,6 +1267,7 @@ handler_again:
 			}
 			break;
 		case ASYNC_STMT_EXECUTE_STORE_RESULT_CONT:
+			PROXY_TRACE2();
 			{ // this copied mostly from ASYNC_USE_RESULT_CONT
 				if (myds->sess && myds->sess->client_myds && myds->sess->mirror==false) {
 					unsigned int buffered_data=0;
@@ -1278,6 +1315,7 @@ handler_again:
 			}
 			break;
 		case ASYNC_STMT_EXECUTE_END:
+			PROXY_TRACE2();
 			{
 				if (query.stmt_result) {
 					unsigned long long total_size=0;
@@ -1346,7 +1384,7 @@ handler_again:
 
 		case ASYNC_NEXT_RESULT_END:
 			break;
-
+#ifndef PROXYSQL_USE_RESULT
 		case ASYNC_STORE_RESULT_START:
 			if (mysql_errno(mysql)) {
 				NEXT_IMMEDIATE(ASYNC_QUERY_END);
@@ -1366,6 +1404,7 @@ handler_again:
 				NEXT_IMMEDIATE(ASYNC_QUERY_END);
 			}
 			break;
+#endif // PROXYSQL_USE_RESULT
 		case ASYNC_USE_RESULT_START:
 			if (mysql_errno(mysql)) {
 				NEXT_IMMEDIATE(ASYNC_QUERY_END);
@@ -1455,6 +1494,7 @@ handler_again:
 			}
 			break;
 		case ASYNC_QUERY_END:
+			PROXY_TRACE2();
 			if (mysql) {
 				int _myerrno=mysql_errno(mysql);
 				if (_myerrno == 0) {
@@ -1602,13 +1642,16 @@ handler_again:
 			break;
 
 		default:
+			// LCOV_EXCL_START
 			assert(0); //we should never reach here
 			break;
+			// LCOV_EXCL_STOP
 		}
 	return async_state_machine;
 }
 
 void MySQL_Connection::process_rows_in_ASYNC_STMT_EXECUTE_STORE_RESULT_CONT(unsigned long long& processed_bytes) {
+	PROXY_TRACE2();
 	// there is more than 1 row
 	unsigned long long total_size=0;
 	long long unsigned int irs = 0;
@@ -1717,7 +1760,9 @@ void MySQL_Connection::next_event(MDB_ASYNC_ST new_st) {
 int MySQL_Connection::async_connect(short event) {
 	PROXY_TRACE();
 	if (mysql==NULL && async_state_machine!=ASYNC_CONNECT_START) {
+		// LCOV_EXCL_START
 		assert(0);
+		// LCOV_EXCL_STOP
 	}
 	if (async_state_machine==ASYNC_IDLE) {
 		myds->wait_until=0;
@@ -1774,6 +1819,7 @@ bool MySQL_Connection::IsServerOffline() {
 // the calling function should check mysql error in mysql struct
 int MySQL_Connection::async_query(short event, char *stmt, unsigned long length, MYSQL_STMT **_stmt, stmt_execute_metadata_t *stmt_meta) {
 	PROXY_TRACE();
+	PROXY_TRACE2();
 	assert(mysql);
 	assert(ret_mysql);
 	server_status=parent->status; // we copy it here to avoid race condition. The caller will see this
@@ -1811,6 +1857,7 @@ int MySQL_Connection::async_query(short event, char *stmt, unsigned long length,
 	}
 	
 	if (async_state_machine==ASYNC_QUERY_END) {
+		PROXY_TRACE2();
 		compute_unknown_transaction_status();
 		if (mysql_errno(mysql)) {
 			return -1;
@@ -1819,6 +1866,7 @@ int MySQL_Connection::async_query(short event, char *stmt, unsigned long length,
 		}
 	}
 	if (async_state_machine==ASYNC_STMT_EXECUTE_END) {
+		PROXY_TRACE2();
 		query.stmt_meta=NULL;
 		async_state_machine=ASYNC_QUERY_END;
 		compute_unknown_transaction_status();
@@ -2144,6 +2192,15 @@ void MySQL_Connection::async_free_result() {
 					mysql_stmt_free_result(query.stmt);
 				}
 			}
+			// If we reached here from 'ASYNC_STMT_PREPARE_FAILED', the
+			// prepared statement was never added to 'local_stmts', thus
+			// it will never be freed when 'local_stmts' are purged. If
+			// initialized, it must be freed. For more context see #3525.
+			if (this->async_state_machine == ASYNC_STMT_PREPARE_FAILED) {
+				if (query.stmt != NULL) {
+					proxy_mysql_stmt_close(query.stmt);
+				}
+			}
 			query.stmt=NULL;
 		}
 		if (mysql_result) {
@@ -2236,13 +2293,18 @@ bool MySQL_Connection::IsKeepMultiplexEnabledVariables(char *query_digest_text)
 	}
 	//filter @@session. and @@
 	char *match=NULL;
+	char* last_pos=NULL;
+	const int at_session_offset = strlen("@@session.");
+	const int double_at_offset = strlen("@@");
 	while (query_digest_text_filter_select && (match = strcasestr(query_digest_text_filter_select,"@@session."))) {
-		*match = '\0';
-		strcat(query_digest_text_filter_select, match+strlen("@@session."));
+		memmove(match, match + at_session_offset, strlen(match) - at_session_offset);
+		last_pos = match + strlen(match) - at_session_offset;
+		*last_pos = '\0';
 	}
 	while (query_digest_text_filter_select && (match = strcasestr(query_digest_text_filter_select,"@@"))) {
-		*match = '\0';
-		strcat(query_digest_text_filter_select, match+strlen("@@"));
+		memmove(match, match + double_at_offset, strlen(match) - double_at_offset);
+		last_pos = match + strlen(match) - double_at_offset;
+		*last_pos = '\0';
 	}
 
 	std::vector<char*>query_digest_text_filter_select_v;
