@@ -167,26 +167,43 @@ enum sslstatus MySQL_Data_Stream::do_ssl_handshake() {
 	int n = SSL_do_handshake(ssl);
 	if (n == 1) {
 		//proxy_info("SSL handshake completed\n");
-		long rc = SSL_get_verify_result(ssl);
-		if (rc != X509_V_OK && rc != X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN && rc != X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE) {
-			proxy_error("Disconnecting %s:%d: X509 client SSL certificate verify error: (%d:%s)\n" , addr.addr, addr.port, rc, X509_verify_cert_error_string(rc));
-			return SSLSTATUS_FAIL;
-		} else {
-			X509 *cert;
-			cert = SSL_get_peer_certificate(ssl);
-			if (cert) {
-				ASN1_STRING *str;
-				GENERAL_NAME *sanName;
-				STACK_OF(GENERAL_NAME) *san_names = NULL;
-				san_names = (stack_st_GENERAL_NAME *)X509_get_ext_d2i((X509 *) cert, NID_subject_alt_name, NULL, NULL);
-				if (san_names) {
-					sanName = sk_GENERAL_NAME_value(san_names, 0);
-					str = sanName->d.dNSName;
-					proxy_info("%s\n" , str->data);
-					x509_subject_alt_name = strdup((const char*)str->data);
+		X509 *cert;
+		cert = SSL_get_peer_certificate(ssl);
+		if (cert) {
+			GENERAL_NAMES *alt_names = (stack_st_GENERAL_NAME *)X509_get_ext_d2i((X509*)cert, NID_subject_alt_name, 0, 0);
+			int alt_name_count = sk_GENERAL_NAME_num(alt_names);
+
+			// Iterate all the SAN names, looking for SPIFFE identifier
+			for (int i = 0; i < alt_name_count; i++) {
+				GENERAL_NAME *san = sk_GENERAL_NAME_value(alt_names, i);
+
+				// We only care about URI names
+				if (san->type == GEN_URI) {
+					if (san->d.uniformResourceIdentifier->data) {
+						const char* resource_data =
+							reinterpret_cast<const char*>(san->d.uniformResourceIdentifier->data);
+						const char* spiffe_loc = strstr(resource_data, "spiffe");
+
+						// First name starting with 'spiffe' is considered the match.
+						if (spiffe_loc == resource_data) {
+							x509_subject_alt_name = strdup(resource_data);
+						}
+					}
 				}
-			} else {
-				proxy_error("X509 error: no required certificate sent by client\n");
+			}
+		} else {
+			// we currently disable this annoying error
+			// in future we can configure this as per user level, specifying if the certificate is mandatory or not
+			// see issue #3424
+			//proxy_error("X509 error: no required certificate sent by client\n");
+		}
+		// In case the supplied certificate has a 'SAN'-'URI' identifier
+		// starting with 'spiffe', client certificate verification is performed.
+		if (x509_subject_alt_name != NULL) {
+			long rc = SSL_get_verify_result(ssl);
+			if (rc != X509_V_OK) {
+				proxy_error("Disconnecting %s:%d: X509 client SSL certificate verify error: (%d:%s)\n" , addr.addr, addr.port, rc, X509_verify_cert_error_string(rc));
+				return SSLSTATUS_FAIL;
 			}
 		}
 	}
@@ -349,6 +366,14 @@ MySQL_Data_Stream::~MySQL_Data_Stream() {
 	}
 	if ( (myconn) && (myds_type==MYDS_FRONTEND) ) { delete myconn; myconn=NULL; }
 	if (encrypted) {
+		if (ssl) {
+			// NOTE: SSL standard requires a final 'close_notify' alert on socket
+			// shutdown. But for avoiding any kind of locking IO waiting for the
+			// other part, we perform a 'quiet' shutdown. For more context see
+			// MYSQL #29579.
+			SSL_set_quiet_shutdown(ssl, 1);
+			SSL_shutdown(ssl);
+		}
 		if (ssl) SSL_free(ssl);
 /*
 		SSL_free() should also take care of these
@@ -428,7 +453,12 @@ void MySQL_Data_Stream::shut_hard() {
 	proxy_debug(PROXY_DEBUG_NET, 4, "Shutdown hard fd=%d. Session=%p, DataStream=%p\n", fd, sess, this);
 	set_net_failure();
 	if (encrypted) {
+		// NOTE: SSL standard requires a final 'close_notify' alert on socket
+		// shutdown. But for avoiding any kind of locking IO waiting for the
+		// other part, we perform a 'quiet' shutdown. For more context see
+		// MYSQL #29579.
 		SSL_set_quiet_shutdown(ssl, 1);
+		SSL_shutdown(ssl);
 	}
 	if (fd >= 0) {
 		shutdown(fd, SHUT_RDWR);
@@ -1308,88 +1338,6 @@ int MySQL_Data_Stream::array2buffer_full() {
 	return rc; 
 }
 
-int MySQL_Data_Stream::myds_connect(char *address, int connect_port, int *pending_connect) {
-	//assert(myconn==NULL);
-
-	assert(myconn); // this assert() is to remove the next condition
-	if (myconn==NULL) myconn= new MySQL_Connection(); // FIXME: why here? // 20141011 /// should be removed
-
-	myconn->last_time_used=sess->thread->curtime;
-	struct sockaddr_un u;
-	struct sockaddr_in a;
-	int s=0;
-	int len=0;
-	int rc=0;
-
-
-	if (connect_port) {
-		// TCP socket
-		if ((s = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
-			perror("socket");
-			close(s);
-			return -1;
-		}
-#ifdef FD_CLOEXEC
-		int f_=fcntl(s, F_GETFL);
-		// asynchronously set also FD_CLOEXEC , this to prevent then when a fork happens the FD are duplicated to new process
-		fcntl(s, F_SETFL, f_|FD_CLOEXEC);
-#endif /* FD_CLOEXEC */
-		ioctl_FIONBIO(s, 1);
-		memset(&a, 0, sizeof(a));
-		a.sin_port = htons(connect_port);
-		a.sin_family = AF_INET;
-
-		if (!inet_aton(address, (struct in_addr *) &a.sin_addr.s_addr)) {
-			perror("bad IP address format");
-			close(s);
-			return -1;
-		}
-	} else {
-		// UNIX socket domain
-		if ((s = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
-			perror("socket");
-			close(s);
-			return -1;
-		}
-#ifdef FD_CLOEXEC
-		int f_=fcntl(s, F_GETFL);
-		// asynchronously set also FD_CLOEXEC , this to prevent then when a fork happens the FD are duplicated to new process
-		fcntl(s, F_SETFL, f_|FD_CLOEXEC);
-#endif /* FD_CLOEXEC */
-		ioctl_FIONBIO(s, 1);
-		memset(u.sun_path,0,UNIX_PATH_MAX);
-		u.sun_family = AF_UNIX;
-		strncpy(u.sun_path, address, UNIX_PATH_MAX-1);
-		len=strlen(u.sun_path)+sizeof(u.sun_family);
-	}
-
-	if (connect_port) {
-		rc=connect(s, (struct sockaddr *) &a, sizeof(a));
-		int arg_on=1;
-		setsockopt(s, IPPROTO_TCP, TCP_NODELAY, (char *) &arg_on, sizeof(int));
-#ifdef __APPLE__
-		setsockopt(s, SOL_SOCKET, SO_NOSIGPIPE, (char *) &arg_on, sizeof(int));
-#endif
-	} else {
-		rc=connect(s, (struct sockaddr *) &u, len);
-	}
-	if (rc==-1) {
-		if (errno!=EINPROGRESS) {
-			perror("connect()");
-			shutdown(s, SHUT_RDWR);
-			close(s);
-			return -1;
-		}
-	} else {
-		*pending_connect=0;
-	}
-	proxy_debug(PROXY_DEBUG_MYSQL_CONNECTION, 5, "Sess=%p, myds=%p, old_mysql_net_fd=%d, fd=%d\n", this->sess, this, myconn->fd, s);
-	myconn->fd=s;
-	proxy_debug(PROXY_DEBUG_MYSQL_CONNECTION, 5, "Sess=%p, myds=%p, new_mysql_net_fd=%d, fd=%d\n", this->sess, this, myconn->fd, s);
-	return s;
-
-}
-
 int MySQL_Data_Stream::assign_fd_from_mysql_conn() {
 	assert(myconn);
 	//proxy_debug(PROXY_DEBUG_MYSQL_CONNECTION, 5, "Sess=%p, myds=%p, oldFD=%d, newFD=%d\n", this->sess, this, fd, myconn->myconn.net.fd);
@@ -1407,11 +1355,6 @@ void MySQL_Data_Stream::unplug_backend() {
   fd=0;
 }
 
-void MySQL_Data_Stream::clean_net_failure() {
-	proxy_debug(PROXY_DEBUG_MYSQL_CONNECTION, 5, "Sess=%p, myds=%p\n", this->sess, this);
-	net_failure=false;
-}
-
 void MySQL_Data_Stream::set_net_failure() {
 	proxy_debug(PROXY_DEBUG_MYSQL_CONNECTION, 5, "Sess=%p, myds=%p , myds_type:%d\n", this->sess, this, myds_type);
 #ifdef DEBUG
@@ -1439,9 +1382,16 @@ void MySQL_Data_Stream::return_MySQL_Connection_To_Pool() {
 	unsigned long long intv = mysql_thread___connection_max_age_ms;
 	intv *= 1000;
 	if (
-		( (intv) && (mc->last_time_used > mc->creation_time + intv) )
+		(( (intv) && (mc->last_time_used > mc->creation_time + intv) )
 		||
-		( mc->local_stmts->get_num_backend_stmts() > (unsigned int)GloMTH->variables.max_stmts_per_connection )
+		( mc->local_stmts->get_num_backend_stmts() > (unsigned int)GloMTH->variables.max_stmts_per_connection ))
+		&&
+		// NOTE: If the current session if in 'PINGING_SERVER' status, there is
+		// no need to reset the session. The destruction and creation of a new
+		// session in case this session has exceeded the time specified by
+		// 'connection_max_age_ms' will be deferred to the next time the session
+		// is used outside 'PINGING_SERVER' operation. For more context see #3502.
+		sess->status != PINGING_SERVER
 	) {
 		if (mysql_thread___reset_connection_algorithm == 2) {
 			sess->create_new_session_and_reset_connection(this);
