@@ -8,6 +8,7 @@
 
 #include "MySQL_Logger.hpp"
 #include "MySQL_Data_Stream.h"
+#include "proxysql_utils.h"
 #include "query_processor.h"
 #include "SQLite3_Server.h"
 
@@ -233,7 +234,80 @@ class sqlite3server_main_loop_listeners {
 
 static sqlite3server_main_loop_listeners S_amll;
 
+/**
+ * @brief Helper function that checks if the supplied string
+ *   is a number.
+ * @param s The string to check.
+ * @return True if the supplied string is just composed of
+ *   digits, false otherwise.
+ */
+bool is_number(const std::string& s) {
+	if (s.empty()) { return false; }
 
+	for (const auto& d : s) {
+		if (std::isdigit(d) == false) {
+			return false;
+		}
+	}
+
+	return true;
+}
+
+/**
+ * @brief Checks if the query matches an specified 'monitor_query' of the
+ *   following format:
+ *
+ *   "$MONITOR_QUERY" + " hostname:port"
+ *
+ *   If the query matches, 'true' is returned, false otherwise.
+ *
+ * @param monitor_query Query that should be matched against the current
+ *   supplied 'query'.
+ * @param query Current query, to be matched against the supplied
+ *   'monitor_query'.
+ * @return 'true' if the query matches, false otherwise.
+ */
+bool match_monitor_query(const std::string& monitor_query, const std::string& query) {
+	if (query.rfind(monitor_query, 0) != 0) {
+		return false;
+	}
+
+	std::string srv_address {
+		query.substr(monitor_query.size())
+	};
+
+	// Check that what is beyond this point, is just the servers address,
+	// written as an identifier 'n.n.n.n:n'.
+	std::size_t cur_mark_pos = 0;
+	for (int i = 0; i < 3; i++) {
+		std::size_t next_mark_pos = srv_address.find('.', cur_mark_pos);
+		if (next_mark_pos == std::string::npos) {
+			return false;
+		} else {
+			std::string number {
+				srv_address.substr(cur_mark_pos, next_mark_pos - cur_mark_pos)
+			};
+
+			if (is_number(number)) {
+				cur_mark_pos = next_mark_pos + 1;
+			} else {
+				return false;
+			}
+		}
+	}
+
+	// Check last part is also a valid number
+	cur_mark_pos = srv_address.find(':', cur_mark_pos);
+	if (cur_mark_pos == std::string::npos) {
+		return false;
+	} else {
+		std::string number {
+			srv_address.substr(cur_mark_pos + 1)
+		};
+
+		return is_number(number);
+	}
+}
 
 void SQLite3_Server_session_handler(MySQL_Session *sess, void *_pa, PtrSize_t *pkt) {
 
@@ -555,8 +629,41 @@ __run_query:
 #ifdef TEST_GROUPREP
 			if (strstr(query_no_space,(char *)"GR_MEMBER_ROUTING_CANDIDATE_STATUS")) {
 				pthread_mutex_lock(&GloSQLite3Server->grouprep_mutex);
-				GloSQLite3Server->populate_grouprep_table(sess, testLag);
-				if (testLag > 0) testLag--;
+				GloSQLite3Server->populate_grouprep_table(sess, 0);
+				// NOTE: This query should be in one place that can be reused by
+				// 'ProxySQL_Monitor' module.
+				const std::string grouprep_monitor_test_query_start {
+					"SELECT viable_candidate,read_only,transactions_behind "
+						"FROM GR_MEMBER_ROUTING_CANDIDATE_STATUS "
+				};
+
+				// If the query matches 'grouprep_monitor_test_query_start', it
+				// means that the query has been issued by `ProxySQL_Monitor` and
+				// we need to fetch for the proper values and replace the query
+				// with one holding the values from `grouprep_map`.
+				if (match_monitor_query(grouprep_monitor_test_query_start, query_no_space)) {
+					std::string srv_addr {
+						query_no_space + grouprep_monitor_test_query_start.size()
+					};
+
+					const group_rep_status& gr_srv_status =
+						GloSQLite3Server->grouprep_test_value(srv_addr);
+					free(query);
+
+					std::string t_select_as_query {
+						"SELECT '%s' AS viable_candidate, '%s' AS read_only, %d AS transactions_behind"
+					};
+					std::string select_as_query {};
+					string_format(
+						t_select_as_query, select_as_query,
+						std::get<0>(gr_srv_status) ? "YES" : "NO",
+						std::get<1>(gr_srv_status) ? "YES" : "NO",
+						std::get<2>(gr_srv_status)
+					);
+
+					query = static_cast<char*>(malloc(select_as_query.length() + 1));
+					strcpy(query, select_as_query.c_str());
+				}
 			}
 #endif // TEST_GROUPREP
 			if (strstr(query_no_space,(char *)"Seconds_Behind_Master")) {
@@ -595,20 +702,12 @@ __run_query:
 #ifdef TEST_GROUPREP
 			if (strstr(query_no_space,(char *)"GR_MEMBER_ROUTING_CANDIDATE_STATUS")) {
 				pthread_mutex_unlock(&GloSQLite3Server->grouprep_mutex);
-				if (resultset->rows_count == 0) {
-					PROXY_TRACE();
-				}
 
-				if (strncmp("127.2.1.2", sess->client_myds->proxy_addr.addr,9) == 0) {
-					if (testTimeoutSequence[testIndex--])
-						sleep(2);
-					if (testIndex < 0)
-						testIndex = 7;
-				}
-				else {
-					if (rand() % 20 == 0)
-						sleep(2);
-				}
+				// NOTE: Enable this just in case of manual testing
+				// if (rand() % 100 == 0) {
+				// 	// randomly add some latency on 1% of the traffic
+				// 	sleep(2);
+				// }
 			}
 #endif // TEST_GROUPREP
 			if (strstr(query_no_space,(char *)"Seconds_Behind_Master")) {
@@ -631,6 +730,18 @@ __run_query:
 	l_free(query_length,query);
 }
 
+#ifdef TEST_GROUPREP
+group_rep_status SQLite3_Server::grouprep_test_value(const std::string& srv_addr) {
+	group_rep_status cur_srv_st { "YES", "YES", 0 };
+
+	auto it = grouprep_map.find(srv_addr);
+	if (it != grouprep_map.end()) {
+		cur_srv_st = it->second;
+	}
+
+	return cur_srv_st;
+}
+#endif
 
 SQLite3_Session::SQLite3_Session() {
 	sessdb=new SQLite3DB();
@@ -705,12 +816,29 @@ static void *child_mysql(void *arg) {
 		if (myds->net_failure) goto __exit_child_mysql;
 		myds->read_pkts();
 		sess->to_process=1;
+
+		// Get and set the client address before the sesion is processed.
+		union {
+			struct sockaddr_in in;
+			struct sockaddr_in6 in6;
+		} custom_sockaddr;
+		struct sockaddr *addr=(struct sockaddr *)malloc(sizeof(custom_sockaddr));
+		socklen_t addrlen=sizeof(custom_sockaddr);
+		memset(addr, 0, sizeof(custom_sockaddr));
+		sess->client_myds->client_addrlen=addrlen;
+		sess->client_myds->client_addr=addr;
+		int g_rc = getpeername(sess->client_myds->fd, addr, &addrlen);
+		if (g_rc == -1) {
+			proxy_error("'getpeername' failed with error: %d\n", rc);
+		}
+
 		int rc=sess->handler();
 		if (rc==-1) goto __exit_child_mysql;
 	}
 
 __exit_child_mysql:
 	delete sqlite_sess;
+	mysql_thr->gen_args = NULL;
 	delete mysql_thr;
 	return NULL;
 }
@@ -765,6 +893,14 @@ static void * sqlite3server_main_loop(void *arg)
 			}
 			fds[i].revents=0;
 		}
+		// NOTE: In case the address imposed by 'sqliteserver-mysql_ifaces' isn't avaible,
+		// a infinite loop could take place if 'POLLNVAL' is not checked here.
+		// This means that trying to set a 'mysql_ifaces' to an address that is
+		// already taken will result into an 'assert' in ProxySQL side.
+		if (nfds == 1 && fds[0].revents == POLLNVAL) {
+			proxy_error("revents==POLLNVAL for FD=%d, events=%d\n", fds[i].fd, fds[i].events);
+			assert(fds[0].revents != POLLNVAL);
+		}
 __end_while_pool:
 		if (S_amll.get_version()!=version) {
 			S_amll.wrlock();
@@ -903,7 +1039,16 @@ void SQLite3_Server::init_grouprep_ifaces_string(std::string& s) {
 	pthread_mutex_init(&grouprep_mutex,NULL);
 	if (!s.empty())
 		s += ";";
-	s += "127.2.1.1:3306;127.2.1.2:3306;127.2.1.3:3306";
+
+	// Maximum number of servers to simulate.
+	max_num_grouprep_servers = 50;
+	for (unsigned int i=0; i < max_num_grouprep_servers; i++) {
+		s += "127.2.1." + std::to_string(i) + ":3306";
+
+		if (i != max_num_grouprep_servers) {
+			s += ";";
+		}
+	}
 }
 #endif // TEST_GROUPREP
 
@@ -1085,19 +1230,77 @@ void SQLite3_Server::populate_aws_aurora_table(MySQL_Session *sess) {
 #endif // TEST_AURORA
 
 #ifdef TEST_GROUPREP
+/**
+ * @brief Populates the 'grouprep' table if it's found empty with the default
+ *   values for the three testing servers.
+ *
+ *   NOTE: This function needs to be called with lock on grouprep_mutex already acquired
+ *
+ * @param sess The current session performing a query.
+ * @param txs_behind Unused parameter.
+ */
 void SQLite3_Server::populate_grouprep_table(MySQL_Session *sess, int txs_behind) {
-	// this function needs to be called with lock on mutex galera_mutex already acquired
-	//
-	sessdb->execute("DELETE FROM GR_MEMBER_ROUTING_CANDIDATE_STATUS");
-	string myip = string(sess->client_myds->proxy_addr.addr);
-	string server_id = myip.substr(8,1);
-	if (server_id == "1")
-		sessdb->execute("INSERT INTO GR_MEMBER_ROUTING_CANDIDATE_STATUS (viable_candidate, read_only, transactions_behind) values ('YES', 'NO', 0)");
-	else {
-		std::stringstream ss;
-		ss << "INSERT INTO GR_MEMBER_ROUTING_CANDIDATE_STATUS (viable_candidate, read_only, transactions_behind) values ('YES', 'YES', " << txs_behind << ")";
-		sessdb->execute(ss.str().c_str());
+	GloAdmin->mysql_servers_wrlock();
+	// We are going to repopulate the map
+	this->grouprep_map.clear();
+
+	char *error=NULL;
+	int cols=0;
+	int affected_rows=0;
+	SQLite3_result *resultset=NULL;
+
+	string query { "SELECT * FROM GR_MEMBER_ROUTING_CANDIDATE_STATUS" };
+	sessdb->execute_statement(query.c_str(), &error, &cols, &affected_rows, &resultset);
+	if (resultset) {
+		for (const SQLite3_row* r : resultset->rows) {
+			std::string srv_addr { std::string(r->fields[0]) + ":" + std::string(r->fields[1]) };
+			const group_rep_status srv_status {
+				std::string { r->fields[2] } == "YES" ? true : false,
+				std::string { r->fields[3] } == "YES" ? true : false,
+				atoi(r->fields[4])
+			};
+
+			this->grouprep_map[srv_addr] = srv_status;
+		}
 	}
+	delete resultset;
+
+	// Insert some default servers for manual testing.
+	//
+	// NOTE: This logic can be improved in the future, for now it only populates
+	// the 'monitoring' data for the default severs. If more servers are placed
+	// as the default ones, more servers will be placed in their appropiated
+	// hostgroups with the same pattern as first ones.
+	if (this->grouprep_map.size() == 0) {
+		GloAdmin->admindb->execute_statement(
+			(char*)"SELECT DISTINCT hostname, port, hostgroup_id FROM mysql_servers"
+			" WHERE hostgroup_id BETWEEN 2700 AND 4200",
+			&error, &cols , &affected_rows , &resultset
+		);
+
+		for (const SQLite3_row* r : resultset->rows) {
+			std::string hostname { r->fields[0] };
+			int port = atoi(r->fields[1]);
+			int hostgroup_id = atoi(r->fields[2]);
+			const std::string t_insert_query {
+				"INSERT INTO GR_MEMBER_ROUTING_CANDIDATE_STATUS"
+					" (hostname, port, viable_candidate, read_only, transactions_behind) VALUES"
+					" ('%s', %d, '%s', '%s', 0)"
+			};
+			std::string insert_query {};
+
+			if (hostgroup_id % 4 == 0) {
+				string_format(t_insert_query, insert_query, hostname.c_str(), port, "YES", "NO");
+				sessdb->execute(insert_query.c_str());
+			} else {
+				string_format(t_insert_query, insert_query, hostname.c_str(), port, "YES", "YES");
+				sessdb->execute(insert_query.c_str());
+			}
+		}
+		delete resultset;
+	}
+
+	GloAdmin->mysql_servers_wrunlock();
 }
 #endif // TEST_GALERA
 
@@ -1170,7 +1373,11 @@ bool SQLite3_Server::init() {
 	tables_defs_grouprep = new std::vector<table_def_t *>;
 	insert_into_tables_defs(tables_defs_grouprep,
 		(const char *)"GR_MEMBER_ROUTING_CANDIDATE_STATUS",
-		(const char*)"CREATE TABLE GR_MEMBER_ROUTING_CANDIDATE_STATUS (viable_candidate varchar not null, read_only varchar not null, transactions_behind int not null)");
+		(const char*)"CREATE TABLE GR_MEMBER_ROUTING_CANDIDATE_STATUS ("
+			"hostname VARCHAR NOT NULL, port INT NOT NULL, viable_candidate varchar not null, read_only varchar not null, transactions_behind int not null, PRIMARY KEY (hostname, port)"
+		")"
+	);
+
 	check_and_build_standard_tables(sessdb, tables_defs_grouprep);
 	GloAdmin->enable_grouprep_testing();
 #endif // TEST_GALERA
