@@ -10,12 +10,31 @@
 #include "re2/regexp.h"
 #include "util/test.h"
 #include "set_parser.h"
+#include <string>
 #include <vector>
 #include <map>
 #include <algorithm>
 #include <iostream>
 
-bool iequals(const string& a, const string& b)
+// *******************************************************************************************
+/**
+ * TODO: This should be fixed once we have improved include hierarchy. All the following
+ * includes are required to avoid the following linker error related to 'GloMyLdapAuth':
+ *
+ * ```
+ * /usr/bin/ld: ../../../lib/libproxysql.a(ProxySQL_GloVars.oo): in function `ProxySQL_GlobalVariables::generate_global_checksum()':
+ * /home/javjarfer/Projects/proxysql_v2.2.0/lib/ProxySQL_GloVars.cpp:374: undefined reference to `GloMyLdapAuth'
+ * ```
+ *
+ * For now we just declare it locally to avoid the linking error.
+ */
+#include "openssl/ssl.h"
+#include "proxysql_structs.h"
+#include "MySQL_LDAP_Authentication.hpp"
+MySQL_LDAP_Authentication *GloMyLdapAuth = nullptr;
+// ******************************************************************************************
+
+bool iequals(const std::string& a, const std::string& b)
 {
     unsigned int sz = a.size();
     if (b.size() != sz)
@@ -88,7 +107,7 @@ static Test sql_mode[] = {
   { "SET sql_mode=(SELCT CONCAT(@@sql_mode, ',PIPES_AS_CONCAT[,NO_ENGINE_SUBSTITUTION'))", {} }
 };
 
-void TestParse(const Test* tests, int ntests, const string& title) {
+void TestParse(const Test* tests, int ntests, const std::string& title) {
   for (int i = 0; i < ntests; i++) {
     std::map<std::string, std::vector<std::string>> data;
     for(auto it = std::begin(tests[i].results); it != std::end(tests[i].results); ++it) {
