@@ -24,6 +24,10 @@ declare SUSER=root
 declare SPASS=
 declare OS_USER=$(whoami)
 
+# Used by the async slaves
+declare REPL_USER="repl_user"
+declare REPL_PASSWORD="pass1234"
+
 # Set this to 1 to run the tests
 declare RUN_TEST=1
 
@@ -38,6 +42,8 @@ declare ALLOW_SHUTDOWN="Yes"
 
 declare PROXYSQL_EXTRA_OPTIONS=""
 
+declare TEST_NAMES=""
+
 declare MYSQL_VERSION
 declare MYSQL_CLIENT_VERSION
 
@@ -82,6 +88,8 @@ Options:
   --proxysql-options=OPTIONS
                       Specify additional options that will be passed
                       to proxysql.
+  --test=test1,test2  Specify a list of comma-separated test names to run.
+
 EOF
 }
 
@@ -119,6 +127,9 @@ function parse_args() {
         --proxysql-options)
           PROXYSQL_EXTRA_OPTIONS=$value
           ;;
+        --test)
+          TEST_NAMES=$value
+          ;;
         *)
           echo "ERROR: unknown parameter \"$param\""
           usage
@@ -157,10 +168,16 @@ function get_mysql_version() {
     echo "5.7"
   elif echo "$mysqld_version" | grep -qe "[[:space:]]8\.0\."; then
     echo "8.0"
+  elif echo "$version_string" | grep -qe "[[:space:]]10\.1\."; then
+    echo "10.1"
   elif echo "$version_string" | grep -qe "[[:space:]]10\.2\."; then
     echo "10.2"
   elif echo "$version_string" | grep -qe "[[:space:]]10\.3\."; then
     echo "10.3"
+  elif echo "$version_string" | grep -qe "[[:space:]]10\.4\."; then
+    echo "10.4"
+  elif echo "$version_string" | grep -qe "[[:space:]]10\.5\."; then
+    echo "10.5"
   else
     echo "Line $LINENO: Cannot determine the MySQL version: $mysqld_version"
     echo "This script needs to be updated."
@@ -617,6 +634,7 @@ echo "Starting ProxySQL..."
 rm -rf $WORKDIR/proxysql_db; mkdir $WORKDIR/proxysql_db
 if [[ ! -r /etc/proxysql.cnf ]]; then
   echo "ERROR! This user($(whoami)) needs read permissions on /etc/proxysql.cnf"
+  echo "something like: 'sudo chmod +r /etc/proxysl.cnf' needs to be run"
   echo "proxysql is started as this user and reads the cnf file"
   echo "This is for TEST purposes and should not be done in PRODUCTION."
   exit 1
@@ -671,11 +689,15 @@ if [[ $MYSQL_VERSION == "5.6" ]]; then
   ${PXC_BASEDIR}/bin/mysql -uroot -S/tmp/cluster_one1.sock <<EOF
 GRANT ALL ON *.* TO admin@'${LOCALHOST_DOMAIN}' identified by 'admin' WITH GRANT OPTION;
 GRANT SELECT ON SYS.* TO monitor@'${LOCALHOST_DOMAIN}' identified by 'monitor';
+-- CREATE USER '${REPL_USER}'@'${LOCALHOST_NAME}' IDENTIFIED BY '${REPL_PASSWORD}';
+-- GRANT REPLICATION SLAVE ON *.* TO '${REPL_USER}'@'${LOCALHOST_NAME}';
 FLUSH PRIVILEGES;
 EOF
 elif [[ $MYSQL_VERSION == "5.7" ]]; then
   ${PXC_BASEDIR}/bin/mysql -uroot -S/tmp/cluster_one1.sock <<EOF
 GRANT ALL ON *.* TO admin@'%' IDENTIFIED BY 'admin' WITH GRANT OPTION;
+-- CREATE USER '${REPL_USER}'@'${LOCALHOST_NAME}' IDENTIFIED BY '${REPL_PASSWORD}';
+-- GRANT REPLICATION SLAVE ON *.* TO '${REPL_USER}'@'${LOCALHOST_NAME}';
 FLUSH PRIVILEGES;
 EOF
 elif [[ $MYSQL_VERSION > "8.0" || $MYSQL_VERSION == "8.0" ]]; then
@@ -683,10 +705,43 @@ elif [[ $MYSQL_VERSION > "8.0" || $MYSQL_VERSION == "8.0" ]]; then
   ${PXC_BASEDIR}/bin/mysql -uroot -S/tmp/cluster_one1.sock <<EOF
 CREATE USER IF NOT EXISTS 'admin'@'%' IDENTIFIED WITH mysql_native_password BY 'admin';
 GRANT ALL ON *.* TO admin@'%' WITH GRANT OPTION;
+-- CREATE USER '${REPL_USER}'@'${LOCALHOST_NAME}' IDENTIFIED WITH mysql_native_password BY '${REPL_PASSWORD}';
+-- GRANT REPLICATION SLAVE ON *.* TO '${REPL_USER}'@'${LOCALHOST_NAME}';
 FLUSH PRIVILEGES;
 EOF
 fi
 
+echo "Starting cluster one async slave..."
+start_async_slave cluster_one 4190
+echo "....cluster one async slave started"
+
+# Setup the slave (note that slave is not started automatically)
+echo "Setting up slave for async replication"
+
+if [[ $MYSQL_VERSION == "5.6" ]]; then
+  ${PXC_BASEDIR}/bin/mysql -S/tmp/cluster_one_slave.sock -uroot <<EOF
+GRANT ALL ON *.* TO admin@'${LOCALHOST_NAME}' identified by 'admin' WITH GRANT OPTION;
+GRANT REPLICATION CLIENT ON *.* TO 'monitor'@'${LOCALHOST_NAME}' IDENTIFIED BY 'monit0r';
+CHANGE MASTER TO MASTER_HOST='$LOCALHOST_IP', MASTER_PORT=4110, MASTER_USER='${REPL_USER}', MASTER_PASSWORD='${REPL_PASSWORD}', MASTER_AUTO_POSITION=1;
+FLUSH PRIVILEGES;
+EOF
+elif [[ $MYSQL_VERSION == "5.7" ]]; then
+  ${PXC_BASEDIR}/bin/mysql -S/tmp/cluster_one_slave.sock -uroot <<EOF
+GRANT ALL ON *.* TO admin@'%' identified by 'admin' WITH GRANT OPTION;
+CHANGE MASTER TO MASTER_HOST='$LOCALHOST_IP', MASTER_PORT=4110, MASTER_USER='${REPL_USER}', MASTER_PASSWORD='${REPL_PASSWORD}', MASTER_AUTO_POSITION=1 FOR CHANNEL 'master-a';
+FLUSH PRIVILEGES;
+EOF
+elif [[ $MYSQL_VERSION > "8.0" || $MYSQL_VERSION == "8.0" ]]; then
+  # For 8.0 separate out the user creation from the grant
+  ${PXC_BASEDIR}/bin/mysql -uroot -S/tmp/cluster_one_slave.sock <<EOF
+CREATE USER IF NOT EXISTS 'admin'@'%' IDENTIFIED WITH mysql_native_password BY 'admin';
+GRANT ALL ON *.* TO admin@'%' WITH GRANT OPTION;
+CHANGE MASTER TO MASTER_HOST='$LOCALHOST_IP', MASTER_PORT=4110, MASTER_USER='${REPL_USER}', MASTER_PASSWORD='${REPL_PASSWORD}', MASTER_AUTO_POSITION=1 FOR CHANNEL 'master-a';
+FLUSH PRIVILEGES;
+EOF
+fi
+
+
 echo "Copying over proxysql-admin.cnf files to /etc"
 if [[ ! -r $PROXYSQL_BASE/etc/proxysql-admin.cnf ]]; then
   echo ERROR! Cannot find $PROXYSQL_BASE/etc/proxysql-admin.cnf
@@ -715,6 +770,7 @@ sudo sed -i "0,/^[ \t]*export OFFLINE_HOSTGROUP_ID[ \t]*=.*$/s|^[ \t]*export OFF
 
 if [[ $RUN_TEST -eq 1 ]]; then
 
+  # Disable existing login.cnf usage
   if [ -e "/dummypathnonexisting/.mylogin.cnf" ]; then
     error "" "/dummypathnonexisting/.mylogin.cnf found. This should not happen.";
     exit 1
@@ -724,7 +780,7 @@ if [[ $RUN_TEST -eq 1 ]]; then
   echo ""
   echo "================================================================"
   echo "proxysql-admin generic bats test log"
-  sudo WORKDIR=$WORKDIR SCRIPTDIR=$SCRIPT_DIR USE_IPVERSION=$USE_IPVERSION \
+  sudo WORKDIR=$WORKDIR SCRIPTDIR=$SCRIPT_DIR USE_IPVERSION=$USE_IPVERSION TEST_NAME=$TEST_NAMES \
         bats $SCRIPT_DIR/generic-test.bats
   echo "================================================================"
   echo ""
@@ -733,7 +789,7 @@ if [[ $RUN_TEST -eq 1 ]]; then
     echo "cluster_one : $test_file"
     SECONDS=0
 
-    sudo WORKDIR=$WORKDIR SCRIPTDIR=$SCRIPT_DIR USE_IPVERSION=$USE_IPVERSION \
+    sudo WORKDIR=$WORKDIR SCRIPTDIR=$SCRIPT_DIR USE_IPVERSION=$USE_IPVERSION TEST_NAME=$TEST_NAMES \
           bats $SCRIPT_DIR/$test_file
     rc=$?
     if (( $SECONDS > 60 )) ; then
@@ -771,6 +827,10 @@ NODES=0
 start_pxc_node cluster_two 4200
 echo "....cluster two started"
 
+echo "Starting cluster two async slave..."
+start_async_slave cluster_two 4290
+echo "....cluster two async slave started"
+
 echo "Creating accounts on the cluster"
 if [[ $MYSQL_VERSION == "5.6" ]]; then
   ${PXC_BASEDIR}/bin/mysql -uroot -S/tmp/cluster_two1.sock <<EOF
@@ -792,6 +852,32 @@ FLUSH PRIVILEGES;
 EOF
 fi
 
+# Setup the slave (note that slave is not started automatically)
+echo "Setting up slave for async replication"
+
+if [[ $MYSQL_VERSION == "5.6" ]]; then
+  ${PXC_BASEDIR}/bin/mysql -S/tmp/cluster_two_slave.sock -uroot <<EOF
+GRANT ALL ON *.* TO admin@'${LOCALHOST_NAME}' identified by 'admin' WITH GRANT OPTION;
+GRANT REPLICATION CLIENT ON *.* TO 'monitor'@'${LOCALHOST_NAME}' IDENTIFIED BY 'monit0r';
+CHANGE MASTER TO MASTER_HOST='$LOCALHOST_IP', MASTER_PORT=4210, MASTER_USER='${REPL_USER}', MASTER_PASSWORD='${REPL_PASSWORD}', MASTER_AUTO_POSITION=1;
+FLUSH PRIVILEGES;
+EOF
+elif [[ $MYSQL_VERSION == "5.7" ]]; then
+  ${PXC_BASEDIR}/bin/mysql -S/tmp/cluster_two_slave.sock -uroot <<EOF
+GRANT ALL ON *.* TO admin@'%' identified by 'admin' WITH GRANT OPTION;
+CHANGE MASTER TO MASTER_HOST='$LOCALHOST_IP', MASTER_PORT=4210, MASTER_USER='${REPL_USER}', MASTER_PASSWORD='${REPL_PASSWORD}', MASTER_AUTO_POSITION=1 FOR CHANNEL 'master-a';
+FLUSH PRIVILEGES;
+EOF
+elif [[ $MYSQL_VERSION > "8.0" || $MYSQL_VERSION == "8.0" ]]; then
+  # For 8.0 separate out the user creation from the grant
+  ${PXC_BASEDIR}/bin/mysql -uroot -S/tmp/cluster_two_slave.sock <<EOF
+CREATE USER IF NOT EXISTS 'admin'@'%' IDENTIFIED WITH mysql_native_password BY 'admin';
+GRANT ALL ON *.* TO admin@'%' WITH GRANT OPTION;
+CHANGE MASTER TO MASTER_HOST='$LOCALHOST_IP', MASTER_PORT=4210, MASTER_USER='${REPL_USER}', MASTER_PASSWORD='${REPL_PASSWORD}', MASTER_AUTO_POSITION=1 FOR CHANNEL 'master-a';
+FLUSH PRIVILEGES;
+EOF
+fi
+
 echo ""
 CLUSTER_TWO_PORT=$(${PXC_BASEDIR}/bin/mysql -uroot -S/tmp/cluster_two1.sock -Bs -e "select @@port")
 sudo sed -i "0,/^[ \t]*export CLUSTER_PORT[ \t]*=.*$/s|^[ \t]*export CLUSTER_PORT[ \t]*=.*$|export CLUSTER_PORT=\"$CLUSTER_TWO_PORT\"|" /etc/proxysql-admin.cnf
@@ -809,7 +895,7 @@ if [[ $RUN_TEST -eq 1 ]]; then
     echo "cluster_two : $test_file"
     SECONDS=0
 
-    sudo WORKDIR=$WORKDIR SCRIPTDIR=$SCRIPT_DIR USE_IPVERSION=$USE_IPVERSION \
+    sudo WORKDIR=$WORKDIR SCRIPTDIR=$SCRIPT_DIR USE_IPVERSION=$USE_IPVERSION TEST_NAME=$TEST_NAMES \
           bats $SCRIPT_DIR/$test_file
     rc=$?
 
